import java.awt.Color;
import java.io.File;
import java.net.URL;
import java.text.DateFormat;

import javax.print.PrintService;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.JobName;
import javax.swing.UIManager;

import jpdf.attribute.*;
import jpdf.gui.dialog.PrintDialog;
import jpdf.gui.dialog.PrintDialogPreferences;
import jpdf.gui.dialog.factory.SetupContentPrintDialogFactory;
import jpdf.gui.preview2.PageLayout;
import jpdf.gui.preview2.PreviewPrintableDialog;
import jpdf.manage.PrintManager;
import jpdf.manage.PrintParameterHolder;
import jpdf.manage.ResourceAdministrator;
import jpdf.printer.EditorPanePrinter;


/**
 * Class PreviewHtmlApplication implements an application which displays the content of an
 * HTML file in a JPDF Preview Dialog.  Page Setup and Print dialogs can be displayed
 * from the Preview Dialog.
 *
 * <P>This class provides an example of how to use the JPDF to implement printing capabilities.
 * The text component is not displayed on the screen to demonstrate the ability to preview and 
 * print off-screen Swing Components.
 */
public class PreviewHtmlApplication
{
  //-----------------------------------------------------------------------
  // Class variables/constants.
  //-----------------------------------------------------------------------
  private static final EditorPanePrinter HTML_PRINTER;
  private static final String            TITLE = "JPDF Information";
  
  
  //-----------------------------------------------------------------------
  // Class Initialization.
  //-----------------------------------------------------------------------
  static
  {
    EditorPanePrinter thePrinter = null;
    URL               theHtmlUrl = null;

    // Set the Look-And-Feel.
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception theEx) {}  
    
    // Create the Printer.
    try
    {
      theHtmlUrl = (new File("resources/JPDF_Demo_Info.html")).toURL();
      thePrinter = EditorPanePrinter.createInstanceForHtml(theHtmlUrl, TITLE);
    }
    catch (Exception theEx) 
    {
      System.out.println("***ERROR: " + theEx);
      System.exit(-99);
    } 
    HTML_PRINTER = thePrinter;
  }
  
  
  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------

  /**
   * Application entry point.
   */
  public static void main(String theParams[]) 
  {
    // Initialize the JPDF Environment.
    initializeJpdfEnvironment();
    
    // Display the Application Window.  
    (new DisplayInfoFrame()).setVisible(true);
  }      

   
  //-----------------------------------------------------------------------
  // Protected methods.
  //-----------------------------------------------------------------------

  /**
   * Displays the Preview Window.
   */
  protected static void displayPreviewWindow(DisplayInfoFrame theParentFrame) 
  {
      PreviewPrintableDialog thePreviewDialog = null;
      
      // Create and display the Preview Dialog.
      // NOTE: The Dialog Factory creates the Page Setup and Print dialogs that can 
      //       be displayed from within the Preview Dialog.
      thePreviewDialog = new PreviewPrintableDialog(theParentFrame, HTML_PRINTER);
      thePreviewDialog.setDialogFactory(new SetupContentPrintDialogFactory()); 
      thePreviewDialog.show();  
  }
  
 /**
   * Displays a Print Dialog which allows the HTML to be printed.
   */
  protected static void displayPrintDialog(DisplayInfoFrame theParentFrame) 
  {
    PrintRequestAttributeSet theAttributeSet = PrintManager.getDefaultPrintAttributes();
    PrintService             thePrintService = null;
    PrintDialog              theDialog = null;
    
    // Create and display the Print Dialog.
    theDialog = new PrintDialog(theParentFrame, theAttributeSet, true); 
    theDialog.show(); 

    // Get the Selected Print Service.
    // NOTE: Null is returned if the user did not press the Print Button.
    thePrintService = theDialog.getSelectedPrintService();
    if (thePrintService != null) 
    {
      // Submit a Print Job.
      PrintManager.printPrintable(HTML_PRINTER, theDialog.getPrintInfoHolder());
    }
  }
  
  /**
   * Exits the application.
   */
  protected static void exitApplication() 
  {
    // Shutdown the JPDF Environment.
    PrintManager.shutdownEnvironment();
    
    // Exit.
    System.exit(0); 
  }
  
  
  //-----------------------------------------------------------------------
  // Private methods.
  //-----------------------------------------------------------------------

  /**
   * Initializes the JPDF environment.
   */
  private static void initializeJpdfEnvironment() 
  {
    final float FOOT_NOTE_FONT_SIZE = 12f;

    PrintRequestAttributeSet  theAttributeSet = null; 
    String                    theDateAndTimeMarker = null;
    String                    thePageAndCountMarker = null;
    String                    thePageHeaderHtml = null;
    
    // Initialize the JPDF Environment.
    try
    {
      PrintManager.initializeEnvironment();      
    }
    catch (Throwable theEx)
    {
      theEx.printStackTrace();
      System.exit(99); 
    }  
    
    // Preload the list of media supported by the Default Print Service on a background thread.  
    // This speeds up the initial display of the Page Setup dialog.
    PrintManager.preloadPrintServiceMediaOnThread();
    
    // Specify the Date Format used in Page Headers/Footers.
    ResourceAdministrator.setTimeStyle(DateFormat.SHORT);
    
    // Print Service selection will be done using icons.
    PrintDialogPreferences.setToUsePrintServiceSelectionIcons(true);

    // Do not include the Banner Page Controller in the Print Dialogs.
    PrintDialogPreferences.setToIncludeBannerPageController(false);

    // Display the Printer name in the Print Dialog title bar.
    PrintDialogPreferences.setToDisplayPrintServiceInTitleBar(true);

    // Set the Preview Dialog preferences.
    PreviewPrintableDialog.setInitialPageLayout(PageLayout.SINGLE_COL);
    PreviewPrintableDialog.setInitialZoomFitPage();
    PreviewPrintableDialog.setPageNavigationAudible(true);
    
    // Get the Default Attribute Set.
    theAttributeSet = PrintManager.getDefaultPrintAttributes(); 
    
    // Set the Page Header HTML.  This HTML specifies the content of the Page Header.  
    // NOTE: The JPDF provides several alternatives for specifying the content of the
    //       Page Headers/Footers.
    thePageHeaderHtml =
    "<HTML>" +
     "<BODY bgcolor='#9999FF'>" +
       "<TABLE cellpadding='0' cellspacing='0' width='100%' style='margin-top: 0px;'>" +
        "<TR bgcolor='#000080'>" + 
          "<TD> " +
            "<IMG SRC='file:./resources/sfw_herald_large.png'>" +
          "</TD> " +
        "</TR>" +
      "</TABLE> " +
     "</BODY>" +
    "</HTML>";

    // Add the initial attributes we want to use for printing.
    // NOTE: Special markers are used in the Page Footer.  These are substituted by actual values
    //       when the material is printed.
    theDateAndTimeMarker = PrintParameterHolder.DATE_MARKER + " " + PrintParameterHolder.TIME_MARKER;
    thePageAndCountMarker = "Page " + PrintParameterHolder.PAGE_NUMBER_MARKER + " of " + 
                            PrintParameterHolder.PAGE_COUNT_MARKER;
    theAttributeSet.add(new JobName("Preview HTML Example", null));
    theAttributeSet.add(new PageHeadHtml(thePageHeaderHtml));   
    theAttributeSet.add(PageHeaderOnFirst2LastPages.FIRST_PAGE_ONLY);
    theAttributeSet.add(new PageFootNotes(theDateAndTimeMarker, "", thePageAndCountMarker, 
                                          Color.BLUE, FOOT_NOTE_FONT_SIZE));
    theAttributeSet.add(new AlignmentFactorHorizontal(0.5f));
    theAttributeSet.add(PageFrame.FALSE);
    theAttributeSet.add(UseBackgroundColor.FALSE);
    theAttributeSet.add(BackgroundColorOnPage.FALSE);    
  }  
}